@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2000 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -60,7 +60,10 @@ intern void pth_debug(char *file, int line, char *fmt, ...)
 
     errno_shield {
         va_start(ap, fmt);
-        sprintf(str, "%d:%s:%04d: ", (int)getpid(), file, line);
+        if (file != NULL)
+            sprintf(str, "%d:%s:%04d: ", (int)getpid(), file, line);
+        else
+            str[0] = NUL;
         n = strlen(str);
         vsprintf(str+n, fmt, ap);
         va_end(ap);
@@ -71,3 +74,38 @@ intern void pth_debug(char *file, int line, char *fmt, ...)
     return;
 }
 
+/* dump out a page to stderr summarizing the internal state of Pth */
+intern void pth_dumpstate(FILE *fp)
+{
+    fprintf(fp, "+----------------------------------------------------------------------\n");
+    fprintf(fp, "| Pth Version: %s\n", PTH_VERSION_STR);
+    fprintf(fp, "| Load Average: %.2f\n", pth_loadval);
+    pth_dumpqueue(fp, "NEW", &pth_NQ);
+    pth_dumpqueue(fp, "READY", &pth_RQ);
+    fprintf(fp, "| Thread Queue RUNNING:\n");
+    fprintf(fp, "|   1. thread 0x%lx (\"%s\")\n", 
+            (unsigned long)pth_current, pth_current->name);
+    pth_dumpqueue(fp, "WAITING", &pth_WQ);
+    pth_dumpqueue(fp, "SUSPENDED", &pth_SQ);
+    pth_dumpqueue(fp, "DEAD", &pth_DQ);
+    fprintf(fp, "+----------------------------------------------------------------------\n");
+    return;
+}
+
+intern void pth_dumpqueue(FILE *fp, char *qn, pth_pqueue_t *q)
+{
+    pth_t t;
+    int n;
+    int i;
+
+    fprintf(fp, "| Thread Queue %s:\n", qn);
+    n = pth_pqueue_elements(q);
+    if (n == 0)
+        fprintf(fp, "|   no threads\n");
+    i = 1;
+    for (t = pth_pqueue_head(q); t != NULL; t = pth_pqueue_walk(q, t, PTH_WALK_NEXT)) {
+        fprintf(fp, "|   %d. thread 0x%lx (\"%s\")\n", i++, (unsigned long)t, t->name);
+    }
+    return;
+}
+
