@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2000 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -29,7 +29,7 @@
 
 #if cpp
 
-/* 
+/*
  * machine context state structure
  *
  * In `jb' the CPU registers, the program counter, the stack
@@ -63,7 +63,7 @@ struct pth_mctx_st {
 ** ____ MACHINE STATE SWITCHING ______________________________________
 */
 
-/* 
+/*
  * save the current machine context
  */
 #if PTH_MCTX_MTH(mcsc)
@@ -83,7 +83,7 @@ struct pth_mctx_st {
 #error "unknown mctx method"
 #endif
 
-/* 
+/*
  * restore the current machine context
  * (at the location of the old context)
  */
@@ -99,8 +99,8 @@ struct pth_mctx_st {
 #error "unknown mctx method"
 #endif
 
-/* 
- * restore the current machine context 
+/*
+ * restore the current machine context
  * (at the location of the new context)
  */
 #if PTH_MCTX_MTH(sjlj) && PTH_MCTX_DSP(sjlje)
@@ -111,16 +111,23 @@ struct pth_mctx_st {
         /*nop*/
 #endif
 
-/* 
- * switch from one machine context to another 
+/*
+ * switch from one machine context to another
  */
+#define SWITCH_DEBUG_LINE \
+        "==== THREAD CONTEXT SWITCH ==========================================="
+#ifdef PTH_DEBUG
+#define  _pth_mctx_switch_debug pth_debug(NULL, 0, SWITCH_DEBUG_LINE);
+#else
+#define  _pth_mctx_switch_debug /*NOP*/
+#endif
 #if PTH_MCTX_MTH(mcsc)
 #define pth_mctx_switch(old,new) \
-    pth_debug1("pth_mctx_switch"); \
+    _pth_mctx_switch_debug \
     swapcontext(&((old)->uc), &((new)->uc));
 #elif PTH_MCTX_MTH(sjlj)
 #define pth_mctx_switch(old,new) \
-    pth_debug1("pth_mctx_switch"); \
+    _pth_mctx_switch_debug \
     if (pth_mctx_save(old) == 0) \
         pth_mctx_restore(new); \
     pth_mctx_restored(old);
@@ -144,7 +151,7 @@ struct pth_mctx_st {
  * for user-space context switching. The thread creation therefore is
  * straight-foreward.
  */
- 
+
 intern int pth_mctx_set(
     pth_mctx_t *mctx, void (*func)(void), char *sk_addr_lo, char *sk_addr_hi)
 {
@@ -166,7 +173,10 @@ intern int pth_mctx_set(
     return TRUE;
 }
 
-#elif PTH_MCTX_MTH(sjlj) && !PTH_MCTX_DSP(sjljlx) && !PTH_MCTX_DSP(sjljisc)
+#elif PTH_MCTX_MTH(sjlj)     &&\
+      !PTH_MCTX_DSP(sjljlx)  &&\
+      !PTH_MCTX_DSP(sjljisc) &&\
+      !PTH_MCTX_DSP(sjljw32)
 
 /*
  * VARIANT 2: THE SIGNAL STACK TRICK
@@ -236,8 +246,8 @@ intern int pth_mctx_set(
 
     pth_debug1("pth_mctx_set: enter");
 
-    /* 
-     * Preserve the SIGUSR1 signal state, block SIGUSR1, 
+    /*
+     * Preserve the SIGUSR1 signal state, block SIGUSR1,
      * and establish our signal handler. The signal will
      * later transfer control onto the signal stack.
      */
@@ -249,7 +259,7 @@ intern int pth_mctx_set(
     sa.sa_flags = SA_ONSTACK;
     if (sigaction(SIGUSR1, &sa, &osa) != 0)
         return FALSE;
-    
+
     /*
      * Set the new stack.
      *
@@ -257,7 +267,7 @@ intern int pth_mctx_set(
      * FreeBSD 3.1]: ``Signal stacks are automatically adjusted
      * for the direction of stack growth and alignment
      * requirements''
-     * 
+     *
      * For sigstack we have to decide ourself [from sigstack(2)
      * on Solaris 2.6]: ``The direction of stack growth is not
      * indicated in the historical definition of struct sigstack.
@@ -265,13 +275,13 @@ intern int pth_mctx_set(
      * the application to determine stack growth direction.''
      */
 #if PTH_MCTX_STK(sas)
-    ss.ss_sp    = pth_skaddr(sigaltstack, sk_addr_lo, sk_addr_hi-sk_addr_lo); 
+    ss.ss_sp    = pth_skaddr(sigaltstack, sk_addr_lo, sk_addr_hi-sk_addr_lo);
     ss.ss_size  = pth_sksize(sigaltstack, sk_addr_lo, sk_addr_hi-sk_addr_lo);
     ss.ss_flags = 0;
     if (sigaltstack(&ss, &oss) < 0)
         return FALSE;
 #elif PTH_MCTX_STK(ss)
-    ss.ss_sp = pth_skaddr(sigstack, sk_addr_lo, sk_addr_hi-sk_addr_lo); 
+    ss.ss_sp = pth_skaddr(sigstack, sk_addr_lo, sk_addr_hi-sk_addr_lo);
     ss.ss_onstack = 0;
     if (sigstack(&ss, &oss) < 0)
         return FALSE;
@@ -279,7 +289,7 @@ intern int pth_mctx_set(
 #error "unknown mctx stack setup"
 #endif
 
-    /* 
+    /*
      * Now transfer control onto the signal stack and set it up.
      * It will return immediately via "return" after the setjmp()
      * was performed. Be careful here with race conditions.  The
@@ -293,7 +303,7 @@ intern int pth_mctx_set(
     while (!mctx_called)
         sigsuspend(&sigs);
 
-    /* 
+    /*
      * Inform the system that we are back off the signal stack by
      * removing the alternative signal stack. Be careful here: It
      * first has to be disabled, before it can be removed.
@@ -304,16 +314,16 @@ intern int pth_mctx_set(
     if (sigaltstack(&ss, NULL) < 0)
         return FALSE;
     sigaltstack(NULL, &ss);
-    if (!(ss.ss_flags & SS_DISABLE)) 
+    if (!(ss.ss_flags & SS_DISABLE))
         return_errno(FALSE, EIO);
-    if (!(oss.ss_flags & SS_DISABLE)) 
+    if (!(oss.ss_flags & SS_DISABLE))
         sigaltstack(&oss, NULL);
 #elif PTH_MCTX_STK(ss)
     if (sigstack(&oss, NULL))
         return FALSE;
 #endif
 
-    /* 
+    /*
      * Restore the old SIGUSR1 signal handler and mask
      */
     sigaction(SIGUSR1, &osa, NULL);
@@ -329,15 +339,15 @@ intern int pth_mctx_set(
     sigemptyset(&mctx->sigs);
     mctx->error = 0;
 
-    /* 
-     * Tell the trampoline and bootstrap function where to dump 
+    /*
+     * Tell the trampoline and bootstrap function where to dump
      * the new machine context, and what to do afterwards...
      */
     mctx_creating      = (volatile pth_mctx_t *)mctx;
     mctx_creating_func = (volatile void (*)(void))func;
     memcpy((void *)&mctx_creating_sigs, &osigs, sizeof(sigset_t));
 
-    /* 
+    /*
      * Now enter the trampoline again, but this time not as a signal
      * handler. Instead we jump into it directly. The functionally
      * redundant ping-pong pointer arithmentic is neccessary to avoid
@@ -357,7 +367,7 @@ intern int pth_mctx_set(
 /* trampoline signal handler */
 static void pth_mctx_set_trampoline(int sig)
 {
-    /* 
+    /*
      * Save current machine state and _immediately_ go back with
      * a standard "return" (to stop the signal handler situation)
      * to let him remove the stack again. Notice that we really
@@ -403,7 +413,7 @@ static void pth_mctx_set_bootstrap(void)
     volatile pth_mctx_t *mctx_starting;
     volatile void (*mctx_starting_func)(void);
 
-    /* 
+    /*
      * Switch to the final signal mask (inherited from parent)
      */
     pth_sc(sigprocmask)(SIG_SETMASK, (sigset_t *)&mctx_creating_sigs, NULL);
@@ -416,14 +426,14 @@ static void pth_mctx_set_bootstrap(void)
     mctx_starting      = mctx_creating;
     mctx_starting_func = mctx_creating_func;
 
-    /* 
-     * Save current machine state (on new stack) and 
+    /*
+     * Save current machine state (on new stack) and
      * go back to caller until we're scheduled for real...
      */
     pth_debug1("pth_mctx_set_trampoline_jumpin: switch back to caller");
     pth_mctx_switch((pth_mctx_t *)mctx_starting, (pth_mctx_t *)&mctx_caller);
 
-    /* 
+    /*
      * The new thread is now running: GREAT!
      * Now we just invoke its init function....
      */
@@ -497,13 +507,37 @@ pth_mctx_set(pth_mctx_t *mctx, void (*func)(void),
     return TRUE;
 }
 
+/*
+ * VARIANT 5: WIN32 SPECIFIC JMP_BUF FIDDLING
+ *
+ * Oh hell, Win32 has setjmp(3), but no sigstack(2) or sigaltstack(2).
+ * So we have to fiddle around with the jmp_buf here too...
+ */
+
+#elif PTH_MCTX_MTH(sjlj) && PTH_MCTX_DSP(sjljw32)
+intern int
+pth_mctx_set(pth_mctx_t *mctx, void (*func)(void),
+                     char *sk_addr_lo, char *sk_addr_hi)
+{
+    pth_mctx_save(mctx);
+#if i386
+    mctx->jb[7] = (int)sk_addr_hi;
+    mctx->jb[8] = (int)func;
+#else
+#error "Unsupported Win32 architecture"
+#endif
+    sigemptyset(&mctx->sigs);
+    mctx->error = 0;
+    return TRUE;
+}
+
 /*
  * VARIANT X: JMP_BUF FIDDLING FOR ONE MORE ESOTERIC OS
  * Add the jmp_buf fiddling for your esoteric OS here...
  *
-#elif PTH_MCTX_MTH(sjlj) && PTH_MCTX_DSP(sjljeso) 
-intern int 
-pth_mctx_set(pth_mctx_t *mctx, void (*func)(void), 
+#elif PTH_MCTX_MTH(sjlj) && PTH_MCTX_DSP(sjljeso)
+intern int
+pth_mctx_set(pth_mctx_t *mctx, void (*func)(void),
              char *sk_addr_lo, char *sk_addr_hi)
 {
     pth_mctx_save(mctx);
