@@ -177,7 +177,7 @@ intern void *pth_scheduler(void *dummy)
     /*
      * endless scheduler loop
      */
-    while (1) {
+    for (;;) {
         /*
          * Move threads from new queue to ready queue and give
          * them maximum priority so they start immediately
