@@ -80,7 +80,7 @@ int pth_mutex_acquire(pth_mutex_t *mutex, int tryonly, pth_event_t ev_extra)
 
     /* else wait for mutex to become unlocked.. */
     pth_debug1("pth_mutex_acquire: wait until mutex is unlocked");
-    while (1) {
+    for (;;) {
         ev = pth_event(PTH_EVENT_MUTEX|PTH_MODE_STATIC, &ev_key, mutex);
         if (ev_extra != NULL)
             pth_event_concat(ev, ev_extra, NULL);
