@@ -92,6 +92,8 @@ int main(int argc, char *argv[])
             fprintf(stderr, "main: read stdin '%c'\n", c);
     }
 
+    pth_cancel(t_ticker);
+    pth_join(t_ticker, NULL);
     pth_event_free(evt, PTH_FREE_THIS);
     pth_kill();
     return 0;
